@@ -725,7 +725,7 @@ test_that("Natural inner join example query #1 returns expected result", {
   skip_if_not(exists("offices") && exists("employees"), message = "Test data not loaded")
   expect_equal(
     query("SELECT * FROM offices NATURAL JOIN employees"),
-    offices %>% inner_join(employees)
+    offices %>% inner_join(employees, multiple = "all")
   )
 })
 
@@ -733,7 +733,7 @@ test_that("Natural left outer join example query #1 returns expected result", {
   skip_if_not(exists("offices") && exists("employees"), message = "Test data not loaded")
   expect_equal(
     query("SELECT * FROM offices NATURAL LEFT OUTER JOIN employees"),
-    offices %>% left_join(employees)
+    offices %>% left_join(employees, multiple = "all")
   )
 })
 
@@ -741,7 +741,7 @@ test_that("Natural right outer join example query #1 returns expected result", {
   skip_if_not(exists("offices") && exists("employees"), message = "Test data not loaded")
   expect_equal(
     query("SELECT * FROM offices NATURAL RIGHT OUTER JOIN employees"),
-    offices %>% right_join(employees)
+    offices %>% right_join(employees, multiple = "all")
   )
 })
 
@@ -749,7 +749,23 @@ test_that("Natural full outer join example query #1 returns expected result", {
   skip_if_not(exists("offices") && exists("employees"), message = "Test data not loaded")
   expect_equal(
     query("SELECT * FROM offices NATURAL FULL OUTER JOIN employees"),
-    offices %>% full_join(employees)
+    offices %>% full_join(employees, multiple = "all")
+  )
+})
+
+test_that("Cross join returns expected result", {
+  skip_if_not(exists("card_rank") && exists("card_suit"), message = "Test data not loaded")
+  expect_equal(
+    query("SELECT rank, suit FROM card_rank CROSS JOIN card_suit;"),
+    if (exists("cross_join", where = "package:dplyr", mode = "function")) {
+      card_rank %>%
+        eval(str2lang("cross_join"))(card_suit) %>%
+        select(rank, suit)
+    } else {
+      card_rank %>%
+        full_join(card_suit, by = character(0)) %>%
+        select(rank, suit)
+    }
   )
 })
 
@@ -777,14 +793,6 @@ test_that("Right semi-join fails", {
   )
 })
 
-test_that("Cross join fails", {
-  skip_if_not(exists("card_rank") && exists("card_suit"), message = "Test data not loaded")
-  expect_error(
-    query("SELECT rank, suit FROM card_rank CROSS JOIN card_suit;"),
-    "Unsupported"
-  )
-})
-
 test_that("Join with three tables fails", {
   skip_if_not(exists("employees") && exists("offices") && exists("orders"), message = "Test data not loaded")
   expect_error(
@@ -824,3 +832,27 @@ test_that("Inner join does not match NAs", {
     2L
   )
 })
+
+
+test_that("Join with one join condition successfully runs with join_by() syntax enabled", {
+  withr::local_options(list(tidyquery.use_join_by = TRUE))
+  skip_if(packageVersion("dplyr") < "1.1.0", message = "Test requires dplyr 1.1.0 or higher")
+  skip_if_not(exists("flights") && exists("airlines"), message = "Test data not loaded")
+  expect_equal(
+    query("SELECT concat_ws(' ', 'Now boarding', name, 'flight', CAST(flight AS string))
+             FROM flights f JOIN airlines a USING (carrier)"),
+    flights %>%
+      inner_join(airlines, by = "carrier") %>%
+      transmute(stringr::str_c("Now boarding", name, "flight", as.character(flight), sep = " "))
+  )
+})
+
+test_that("Join with two join conditions successfully runs with join_by() syntax enabled", {
+  withr::local_options(list(tidyquery.use_join_by = TRUE))
+  skip_if(packageVersion("dplyr") < "1.1.0", message = "Test requires dplyr 1.1.0 or higher")
+  skip_if_not(exists("inventory") && exists("games"), message = "Test data not loaded")
+  expect_equal(
+    query("SELECT * FROM inventory INNER JOIN games ON inventory.game = games.name AND inventory.price = games.list_price"),
+    inventory %>% inner_join(games, by = c(game = "name", price = "list_price"))
+  )
+})
